@@ -26,10 +26,6 @@
 import os
 
 import bzrlib
-from bzrlib import (
-    merge,
-    msgeditor,
-    )
 from bzrlib.commands import plugin_cmds
 from bzrlib.directory_service import directories
 
@@ -38,24 +34,13 @@ from info import (
     )
 
 
-if getattr(merge, 'ConfigurableFileMerger', None) is None:
-    raise ImportError(
-        'need at least bzr 2.1.0rc2 (you use %r)', bzrlib.version_info)
-else:
-    def changelog_merge_hook_factory(merger):
-        from bzrlib.plugins.builddeb import merge_changelog
-        return merge_changelog.ChangeLogFileMerge(merger)
-
-    merge.Merger.hooks.install_named_hook(
-        'merge_file_content', changelog_merge_hook_factory,
-        'Debian Changelog file merge')
-
-
 commands = {
         "bd_do": [],
         "builddeb": ["bd"],
+        "dep3_patch": [],
         "dh_make": ["dh_make"],
         "import_dsc": [],
+        "import_upstream": [],
         "mark_uploaded": [],
         "merge_package": [],
         "merge_upstream": ["mu"],
@@ -67,7 +52,9 @@ for command, aliases in commands.iteritems():
 
 builddeb_dir = '.bzr-builddeb'
 default_conf = os.path.join(builddeb_dir, 'default.conf')
-global_conf = os.path.expanduser('~/.bazaar/builddeb.conf')
+def global_conf():
+    from bzrlib.config import config_dir
+    return os.path.join(config_dir(), 'builddeb.conf')
 local_conf = os.path.join(builddeb_dir, 'local.conf')
 
 default_build_dir = '../build-area'
@@ -75,8 +62,8 @@ default_orig_dir = '..'
 default_result_dir = '..'
 
 
-directories.register_lazy("deb:", 'bzrlib.plugins.builddeb.directory', 
-        'VcsDirectory', 
+directories.register_lazy("apt:", 'bzrlib.plugins.builddeb.directory',
+        'VcsDirectory',
         "Directory that uses Debian Vcs-* control fields to look up branches")
 
 
@@ -123,11 +110,96 @@ def debian_changelog_commit_message(commit, start_message):
     return "".join(changes)
 
 
-msgeditor.hooks.install_named_hook("commit_message_template",
-        debian_changelog_commit_message,
-        "Use changes documented in debian/changelog to suggest "
-        "the commit message")
+def debian_changelog_commit(commit, start_message):
+    """hooked into bzrlib.msgeditor set_commit_message.
+     Set the commit message from debian/changelog and set any LP: #1234 to bug
+     fixed tags."""
+    from bzrlib.plugins.builddeb.util import find_bugs_fixed
 
+    changes = debian_changelog_commit_message(commit, start_message)
+    if changes is None:
+        return None
+
+    bugs_fixed = find_bugs_fixed([changes], commit.work_tree.branch)
+    commit.builder._revprops["bugs"] = "\n".join(bugs_fixed)
+
+    return debian_changelog_commit_message(commit, start_message)
+
+
+def changelog_merge_hook_factory(merger):
+    from bzrlib.plugins.builddeb import merge_changelog
+    return merge_changelog.ChangeLogFileMerge(merger)
+
+
+def debian_tag_name(branch, revid):
+    from bzrlib.plugins.builddeb.config import BUILD_TYPE_MERGE
+    from bzrlib.plugins.builddeb.errors import MissingChangelogError
+    from bzrlib.plugins.builddeb.import_dsc import (DistributionBranch,
+        DistributionBranchSet)
+    from bzrlib.plugins.builddeb.util import (debuild_config, find_changelog)
+    t = branch.repository.revision_tree(revid)
+    config = debuild_config(t, False)
+    try:
+        (changelog, larstiq) = find_changelog(t, config.build_type == BUILD_TYPE_MERGE)
+    except MissingChangelogError:
+        # Not a debian package
+        return None
+    if changelog.distributions == 'UNRELEASED':
+        # The changelog still targets 'UNRELEASED', so apparently hasn't been 
+        # uploaded. XXX: Give a warning of some sort here?
+        return None
+    db = DistributionBranch(branch, None)
+    dbs = DistributionBranchSet()
+    dbs.add_branch(db)
+    return db.tag_name(changelog.version)
+
+
+try:
+    from bzrlib.hooks import install_lazy_named_hook
+except ImportError: # Compatibility with bzr < 2.4
+    from bzrlib import (
+        branch as _mod_branch,
+        errors,
+        merge,
+        msgeditor,
+        )
+    msgeditor.hooks.install_named_hook("commit_message_template",
+            debian_changelog_commit_message,
+            "Use changes documented in debian/changelog to suggest "
+            "the commit message")
+    if getattr(merge, 'ConfigurableFileMerger', None) is None:
+        raise ImportError(
+            'need at least bzr 2.1.0rc2 (you use %r)', bzrlib.version_info)
+    else:
+        merge.Merger.hooks.install_named_hook(
+            'merge_file_content', changelog_merge_hook_factory,
+            'Debian Changelog file merge')
+    try:
+        _mod_branch.Branch.hooks.install_named_hook("automatic_tag_name",
+             debian_tag_name,
+             "Automatically determine tag names from Debian version")
+    except errors.UnknownHook:
+        pass # bzr < 2.2 doesn't have this hook.
+else:
+    install_lazy_named_hook(
+        "bzrlib.msgeditor", "hooks", "commit_message_template",
+            debian_changelog_commit_message,
+            "Use changes documented in debian/changelog to suggest "
+            "the commit message")
+    if bzrlib.version_info[0] >= 2 and bzrlib.version_info[1] >= 4:
+        install_lazy_named_hook(
+            "bzrlib.msgeditor", "hooks", "set_commit_message",
+                debian_changelog_commit,
+                "Use changes documented in debian/changelog to set "
+                "the commit message and bugs fixed")
+    install_lazy_named_hook(
+        "bzrlib.merge", "Merger.hooks",
+        'merge_file_content', changelog_merge_hook_factory,
+        'Debian Changelog file merge')
+    install_lazy_named_hook(
+        "bzrlib.branch", "Branch.hooks",
+        "automatic_tag_name", debian_tag_name,
+         "Automatically determine tag names from Debian version")
 
 try:
     from bzrlib.revisionspec import revspec_registry
@@ -138,6 +210,15 @@ except ImportError:
     from bzrlib.plugins.builddeb.revspec import RevisionSpec_package
     SPEC_TYPES.append(RevisionSpec_package)
 
+try:
+    from bzrlib.tag import tag_sort_methods
+except ImportError:
+    pass # bzr tags --sort= can not be extended
+else:
+    tag_sort_methods.register_lazy("debversion",
+        "bzrlib.plugins.builddeb.tagging", "sort_debversion",
+        "Sort like Debian versions.")
+
 
 def load_tests(standard_tests, module, loader):
     return loader.loadTestsFromModuleNames(['bzrlib.plugins.builddeb.tests'])
