@@ -18,12 +18,18 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #
 
-from bzrlib.config import ConfigObj, TreeConfig
+from bzrlib.config import (
+  configobj,
+  ConfigObj,
+  TreeConfig,
+  )
 from bzrlib.trace import mutter, warning
-try:
-  from bzrlib.util.configobj.configobj import ParseError
-except ImportError:
-  from configobj import ParseError
+
+
+BUILD_TYPE_NORMAL = "normal"
+BUILD_TYPE_NATIVE = "native"
+BUILD_TYPE_MERGE = "merge"
+BUILD_TYPE_SPLIT = "split"
 
 
 class SvnBuildPackageMappedConfig(object):
@@ -94,7 +100,7 @@ class DebBuildConfig(object):
     for input in files:
       try:
         config = ConfigObj(input[0])
-      except ParseError, e:
+      except configobj.ParseError, e:
         if len(input) > 2:
           content = input[2]
         else:
@@ -257,6 +263,17 @@ class DebBuildConfig(object):
 
   merge = _bool_property('merge', "Run in merge mode")
 
+  @property
+  def build_type(self):
+    if self.merge:
+      return BUILD_TYPE_MERGE
+    elif self.native:
+      return BUILD_TYPE_NATIVE
+    elif self.split:
+      return BUILD_TYPE_SPLIT
+    else:
+      return None
+
   quick_builder = _opt_property('quick-builder',
                           "A quick command to build with", True)
 
