@@ -24,6 +24,7 @@ except ImportError:
     import md5
 import signal
 import shutil
+import subprocess
 import tempfile
 import os
 import re
@@ -39,11 +40,11 @@ except ImportError:
     from debian_bundle.changelog import Changelog, ChangelogParseError
 
 from bzrlib import (
-        bugtracker,
-        errors,
-        urlutils,
-        version_info as bzr_version_info,
-        )
+    bugtracker,
+    errors,
+    urlutils,
+    version_info as bzr_version_info,
+    )
 from bzrlib.export import export as bzr_export
 from bzrlib.trace import warning
 from bzrlib.transport import (
@@ -55,23 +56,29 @@ from bzrlib.plugins.builddeb import (
     local_conf,
     global_conf,
     )
-from bzrlib.plugins.builddeb.config import DebBuildConfig
+from bzrlib.plugins.builddeb.config import (
+    DebBuildConfig,
+    BUILD_TYPE_MERGE,
+    BUILD_TYPE_NATIVE,
+    BUILD_TYPE_NORMAL,
+    )
 from bzrlib.plugins.builddeb.errors import (
-                MissingChangelogError,
-                AddChangelogError,
-                NoPreviousUpload,
-                UnableToFindPreviousUpload,
-                UnknownDistribution,
-                UnparseableChangelog,
-                )
+    MissingChangelogError,
+    AddChangelogError,
+    InconsistentSourceFormatError,
+    NoPreviousUpload,
+    TarFailed,
+    UnableToFindPreviousUpload,
+    UnparseableChangelog,
+    )
 
 
-DEBIAN_RELEASES = ('woody', 'sarge', 'etch', 'lenny', 'squeeze', 'stable',
-        'testing', 'unstable', 'experimental', 'frozen', 'sid')
+DEBIAN_RELEASES = ('woody', 'sarge', 'etch', 'lenny', 'squeeze', 'wheezy',
+        'stable', 'testing', 'unstable', 'experimental', 'frozen', 'sid')
 DEBIAN_POCKETS = ('', '-security', '-proposed-updates', '-backports')
 UBUNTU_RELEASES = ('warty', 'hoary', 'breezy', 'dapper', 'edgy',
         'feisty', 'gutsy', 'hardy', 'intrepid', 'jaunty', 'karmic',
-        'lucid', 'maverick')
+        'lucid', 'maverick', 'natty', "oneiric")
 UBUNTU_POCKETS = ('', '-proposed', '-updates', '-security', '-backports')
 
 
@@ -140,7 +147,7 @@ def find_changelog(t, merge, max_blocks=1):
     try:
         if not t.has_filename(changelog_file):
             if merge:
-                #Assume LarstiQ's layout (.bzr in debian/)
+                # Assume LarstiQ's layout (.bzr in debian/)
                 changelog_file = 'changelog'
                 larstiq = True
                 if not t.has_filename(changelog_file):
@@ -149,10 +156,12 @@ def find_changelog(t, merge, max_blocks=1):
             else:
                 raise MissingChangelogError('"debian/changelog"')
         elif merge and t.has_filename('changelog'):
-            # If it is a "larstiq" pacakge and debian is a symlink to
+            # If it is a "larstiq" package and debian is a symlink to
             # "." then it will have found debian/changelog. Try and detect
             # this.
-            if (t.kind(t.path2id('debian')) == 'symlink' and 
+            debian_file_id = t.path2id('debian')
+            if (debian_file_id is not None and 
+                t.kind(debian_file_id) == 'symlink' and 
                 t.get_symlink_target(t.path2id('debian')) == '.'):
                 changelog_file = 'changelog'
                 larstiq = True
@@ -200,53 +209,22 @@ def strip_changelog_message(changes):
         return changes
 
 
-def tarball_name(package, version, format=None):
+def tarball_name(package, version, component=None, format=None):
     """Return the name of the .orig.tar.gz for the given package and version.
 
     :param package: the name of the source package.
     :param version: the upstream version of the package.
+    :param component: Component name (None for base)
     :param format: the format for the tarball. If None then 'gz' will be
          used. You probably want on of 'gz', 'bz2', or 'lzma'.
     :return: a string that is the name of the upstream tarball to use.
     """
     if format is None:
         format = 'gz'
-    return "%s_%s.orig.tar.%s" % (package, str(version), format)
-
-
-def get_snapshot_revision(upstream_version):
-    """Return the upstream revision specifier if specified in the upstream version.
-
-    When packaging an upstream snapshot some people use +vcsnn or ~vcsnn to indicate
-    what revision number of the upstream VCS was taken for the snapshot. This given
-    an upstream version number this function will return an identifier of the
-    upstream revision if it appears to be a snapshot. The identifier is a string
-    containing a bzr revision spec, so it can be transformed in to a revision.
-
-    :param upstream_version: a string containing the upstream version number.
-    :return: a string containing a revision specifier for the revision of the
-        upstream branch that the snapshot was taken from, or None if it doesn't
-        appear to be a snapshot.
-    """
-    match = re.search("(?:~|\\+)bzr([0-9]+)$", upstream_version)
-    if match is not None:
-        return match.groups()[0]
-    match = re.search("(?:~|\\+)svn([0-9]+)$", upstream_version)
-    if match is not None:
-        return "svn:%s" % match.groups()[0]
-    return None
-
-
-def get_export_upstream_revision(config, version=None):
-    rev = None
-    if version is not None:
-        rev = get_snapshot_revision(str(version.upstream_version))
-    if rev is None:
-        rev = config._get_best_opt('export-upstream-revision')
-        if rev is not None and version is not None:
-            rev = rev.replace('$UPSTREAM_VERSION',
-                              str(version.upstream_version))
-    return rev
+    name = "%s_%s.orig" % (package, str(version))
+    if component is not None:
+        name += "-" + component
+    return "%s.tar.%s" % (name, format)
 
 
 def suite_to_distribution(suite):
@@ -257,7 +235,8 @@ def suite_to_distribution(suite):
     Ubuntu).
 
     :param suite: the string containing the suite
-    :return: "debian", "ubuntu", or None if the distribution couldn't be inferred.
+    :return: "debian", "ubuntu", or None if the distribution couldn't be
+        inferred.
     """
     all_debian = [r + t for r in DEBIAN_RELEASES for t in DEBIAN_POCKETS]
     all_ubuntu = [r + t for r in UBUNTU_RELEASES for t in UBUNTU_POCKETS]
@@ -281,7 +260,11 @@ def lookup_distribution(distribution_or_suite):
 
 
 def md5sum_filename(filename):
-    """Calculate the md5sum of a file by name."""
+    """Calculate the md5sum of a file by name.
+
+    :param filename: Path of the file to checksum
+    :return: MD5 Checksum as hex digest
+    """
     m = md5.md5()
     f = open(filename, 'rb')
     try:
@@ -293,6 +276,12 @@ def md5sum_filename(filename):
 
 
 def move_file_if_different(source, target, md5sum):
+    """Overwrite a file if its new contents would be different from the current contents.
+
+    :param source: Path of the source file
+    :param target: Path of the target file
+    :param md5sum: MD5Sum (as hex digest) of the source file
+    """
     if os.path.exists(target):
         if os.path.samefile(source, target):
             return
@@ -303,6 +292,11 @@ def move_file_if_different(source, target, md5sum):
 
 
 def write_if_different(contents, target):
+    """(Over)write a file with `contents` if they are different from its current content.
+
+    :param contents: The contents to write, as a string
+    :param target: Path of the target file
+    """
     md5sum = md5.md5()
     md5sum.update(contents)
     fd, temp_path = tempfile.mkstemp("builddeb-rename-")
@@ -326,7 +320,7 @@ def _download_part(name, base_transport, target_dir, md5sum):
     f_f = f_t.get(part_path)
     try:
         target_path = os.path.join(target_dir, part_path)
-        fd, temp_path = tempfile.mkstemp(prefix="builldeb-")
+        fd, temp_path = tempfile.mkstemp(prefix="builddeb-")
         fobj = os.fdopen(fd, "wb")
         try:
             try:
@@ -341,29 +335,34 @@ def _download_part(name, base_transport, target_dir, md5sum):
         f_f.close()
 
 
-def open_file(path):
-    filename, transport = open_transport(path)
+def open_file(url):
+    """Open a file from a URL.
+
+    :param url: URL to open
+    :return: A file-like object.
+    """
+    filename, transport = open_transport(url)
     return open_file_via_transport(filename, transport)
 
 
 def open_transport(path):
-  """Obtain an appropriate transport instance for the given path."""
-  base_dir, path = urlutils.split(path)
-  transport = get_transport(base_dir)
-  return (path, transport)
+    """Obtain an appropriate transport instance for the given path."""
+    base_dir, path = urlutils.split(path)
+    transport = get_transport(base_dir)
+    return (path, transport)
 
 
 def open_file_via_transport(filename, transport):
-  """Open a file using the transport, follow redirects as necessary."""
-  def open_file(transport):
-    return transport.get(filename)
-  def follow_redirection(transport, e, redirection_notice):
-    mutter(redirection_notice)
-    _filename, redirected_transport = open_transport(e.target)
-    return redirected_transport
+    """Open a file using the transport, follow redirects as necessary."""
+    def open_file(transport):
+        return transport.get(filename)
+    def follow_redirection(transport, e, redirection_notice):
+        mutter(redirection_notice)
+        _filename, redirected_transport = open_transport(e.target)
+        return redirected_transport
 
-  result = do_catching_redirections(open_file, transport, follow_redirection)
-  return result
+    result = do_catching_redirections(open_file, transport, follow_redirection)
+    return result
 
 
 def _dget(cls, dsc_location, target_dir):
@@ -396,6 +395,13 @@ def get_parent_dir(target):
 
 
 def find_bugs_fixed(changes, branch, _lplib=None):
+    """Find the bugs marked fixed in a changelog entry.
+
+    :param changes: A list of the contents of the changelog entry.
+    :param branch: Bazaar branch associated with the package
+    :return: String with bugs closed, as appropriate for a Bazaar "bugs" revision 
+        property.
+    """
     if _lplib is None:
         from bzrlib.plugins.builddeb import launchpad as _lplib
     bugs = []
@@ -405,30 +411,30 @@ def find_bugs_fixed(changes, branch, _lplib=None):
                 re.IGNORECASE):
             closes_list = match.group(0)
             for match in re.finditer("\d+", closes_list):
-                bug_url = bugtracker.get_bug_url("deb", branch,
-                        match.group(0))
+                bug_url = bugtracker.get_bug_url("deb", branch, match.group(0))
                 bugs.append(bug_url + " fixed")
                 lp_bugs = _lplib.ubuntu_bugs_for_debian_bug(match.group(0))
                 if len(lp_bugs) == 1:
-                    bug_url = bugtracker.get_bug_url("lp", branch,
-                            lp_bugs[0])
+                    bug_url = bugtracker.get_bug_url("lp", branch, lp_bugs[0])
                     bugs.append(bug_url + " fixed")
         for match in re.finditer("lp:\s+\#\d+(?:,\s*\#\d+)*",
                 change, re.IGNORECASE):
             closes_list = match.group(0)
             for match in re.finditer("\d+", closes_list):
-                bug_url = bugtracker.get_bug_url("lp", branch,
-                        match.group(0))
+                bug_url = bugtracker.get_bug_url("lp", branch, match.group(0))
                 bugs.append(bug_url + " fixed")
                 deb_bugs = _lplib.debian_bugs_for_ubuntu_bug(match.group(0))
                 if len(deb_bugs) == 1:
-                    bug_url = bugtracker.get_bug_url("deb", branch,
-                            deb_bugs[0])
+                    bug_url = bugtracker.get_bug_url("deb", branch, deb_bugs[0])
                     bugs.append(bug_url + " fixed")
     return bugs
 
 
 def find_extra_authors(changes):
+    """Find additional authors from a changelog entry.
+
+    :return: List of fullnames of additional authors, without e-mail address.
+    """
     extra_author_re = re.compile(r"\s*\[([^\]]+)]\s*")
     authors = []
     for change in changes:
@@ -447,6 +453,11 @@ def find_extra_authors(changes):
 
 
 def find_thanks(changes):
+    """Find all people thanked in a changelog entry.
+
+    :param changes: String with the contents of the changelog entry
+    :return: List of people thanked, optionally including email address.
+    """
     thanks_re = re.compile(r"[tT]hank(?:(?:s)|(?:you))(?:\s*to)?"
             "((?:\s+(?:(?:\w\.)|(?:\w+(?:-\w+)*)))+"
             "(?:\s+<[^@>]+@[^@>]+>)?)",
@@ -579,8 +590,13 @@ def find_previous_upload(tree, merge):
 
 
 def _find_previous_upload(cl):
-    blocks = cl._blocks
-    current_target = blocks[0].distributions.split(" ")[0]
+    """Find the version of the previous upload.
+
+    :param cl: Changelog object
+    :return: Version object for the previous upload
+    :raise NoPreviousUpload: Raised when there is no previous upload
+    """
+    current_target = find_last_distribution(cl)
     all_debian = [r + t for r in DEBIAN_RELEASES for t in DEBIAN_POCKETS]
     all_ubuntu = [r + t for r in UBUNTU_RELEASES for t in UBUNTU_POCKETS]
     if current_target in all_debian:
@@ -588,12 +604,15 @@ def _find_previous_upload(cl):
     elif current_target in all_ubuntu:
         match_targets = UBUNTU_RELEASES
         if "-" in current_target:
-            match_targets += [current_target.split("-", 1)[0]
-                + t for t in UBUNTU_POCKETS]
+            match_targets += tuple([current_target.split("-", 1)[0]
+                + t for t in UBUNTU_POCKETS])
     else:
-        raise UnknownDistribution(current_target)
+        # If we do not recognize the current target in order to apply special
+        # rules to it, then just assume that only previous uploads to exactly
+        # the same target count.
+        match_targets = (current_target,)
     previous_version = None
-    for block in blocks[1:]:
+    for block in cl._blocks[1:]:
         if block.distributions.split(" ")[0] in match_targets:
             return block.version
     raise NoPreviousUpload(current_target)
@@ -601,11 +620,136 @@ def _find_previous_upload(cl):
 
 def tree_contains_upstream_source(tree):
     """Guess if the specified tree contains the upstream source.
-    
+
     :param tree: A RevisionTree.
     :return: Boolean indicating whether or not the tree contains the upstream
-        source
+        source. None if the tree is empty
     """
-    present_files = set(tree.inventory.root.children.keys())
-    packaging_files = frozenset(["debian", ".bzr-builddeb"])
+    present_files = set(
+        [f[0] for f in tree.list_files(recursive=False)
+         if f[1] == 'V'])
+    if len(present_files) == 0:
+        return None
+    packaging_files = frozenset(["debian", ".bzr-builddeb", ".bzrignore"])
     return (len(present_files - packaging_files) > 0)
+
+
+def get_source_format(tree):
+    """Retrieve the source format name from a package.
+
+    :param path: Path to the package
+    :return: String with package format
+    """
+    filename = "debian/source/format"
+    if not tree.has_filename(filename):
+        return FORMAT_1_0
+    text = tree.get_file_text(tree.path2id(filename), filename)
+    return text.strip()
+
+
+FORMAT_1_0 = "1.0"
+FORMAT_3_0_QUILT = "3.0 (quilt)"
+FORMAT_3_0_NATIVE = "3.0 (native)"
+
+NATIVE_SOURCE_FORMATS = [FORMAT_3_0_NATIVE]
+NORMAL_SOURCE_FORMATS = [FORMAT_3_0_QUILT]
+
+
+def guess_build_type(tree, version, contains_upstream_source):
+    """Guess the build type based on the contents of a tree.
+
+    :param tree: A `Tree` object.
+    :param version: `Version` of the upload.
+    :param contains_upstream_source: Whether this branch contains the upstream source.
+    :return: A build_type value.
+    """
+    source_format = get_source_format(tree)
+    if source_format in NATIVE_SOURCE_FORMATS:
+        format_native = True
+    elif source_format in NORMAL_SOURCE_FORMATS:
+        format_native = False
+    else:
+        format_native = None
+
+    # If the package doesn't have a debian revision then it must be native.
+    if version is not None:
+        version_native = (not version.debian_version)
+    else:
+        version_native = None
+
+    if type(version_native) is bool and type(format_native) is bool:
+        if version_native != format_native:
+            raise InconsistentSourceFormatError(version_native, format_native)
+
+    if version_native or format_native:
+        return BUILD_TYPE_NATIVE
+    if contains_upstream_source == False:
+        # Default to merge mode if there's only a debian/ directory
+        return BUILD_TYPE_MERGE
+    else:
+        return BUILD_TYPE_NORMAL
+
+
+def component_from_orig_tarball(tarball_filename, package, version):
+    tarball_filename = os.path.basename(tarball_filename)
+    prefix = "%s_%s.orig" % (package, version)
+    if not tarball_filename.startswith(prefix):
+        raise ValueError(
+            "invalid orig tarball file %s does not have expected prefix %s" % (
+                tarball_filename, prefix))
+    base = tarball_filename[len(prefix):]
+    for ext in (".tar.gz", ".tar.bz2", ".tar.lzma"):
+        if tarball_filename.endswith(ext):
+            base = base[:-len(ext)]
+            break
+    else:
+        raise ValueError(
+            "orig tarball file %s has unknown extension" % tarball_filename)
+    if base == "":
+        return None
+    elif base[0] == "-":
+        # Extra component
+        return base[1:]
+    else:
+        raise ValueError("Invalid extra characters in tarball filename %s" %
+            tarball_filename)
+
+
+def extract_orig_tarball(tarball_filename, component, target, strip_components=None):
+    """Extract an orig tarball.
+
+    :param tarball: Path to the tarball
+    :param component: Component name (or None for top-level)
+    :param target: Target path
+    :param strip_components: Optional number of components to strip
+    """
+    tar_args = ["tar"]
+    if tarball_filename.endswith(".tar.bz2"):
+        tar_args.append('xjf')
+    elif tarball_filename.endswith(".tar.lzma"):
+        tar_args.append('xJf')
+    else:
+        tar_args.append('xzf')
+    if component is not None:
+        target_path = os.path.join(target, component)
+        os.mkdir(target_path)
+    else:
+        target_path = target
+    tar_args.extend([tarball_filename, "-C", target_path])
+    if strip_components is not None:
+        tar_args.extend(["--strip-components", "1"])
+    proc = subprocess.Popen(tar_args, preexec_fn=subprocess_setup)
+    proc.communicate()
+    if proc.returncode != 0:
+        raise TarFailed("extract", tarball_filename)
+
+
+def extract_orig_tarballs(tarballs, target, strip_components=None):
+    """Extract orig tarballs to a directory.
+
+    :param tarballs: List of tarball filenames
+    :param target: Target directory (must already exist)
+    """
+    for tarball_filename, component in tarballs:
+        extract_orig_tarball(tarball_filename, component, target,
+            strip_components=strip_components)
